@@ -8,9 +8,6 @@ import sys
 import os
 import locale
 
-e = os.environ.get('SIGIL_QT_RUNTIME_VERSION', '5.10.0')
-SIGIL_QT_MAJOR_VERSION = tuple(map(int, (e.split("."))))[0]
-
 def run(bk):
     # get current epub version
     epubversion = "2.0"
@@ -30,29 +27,16 @@ def run(bk):
     print(" ")
     print("Verifying proper Python packages are available")
 
-    if SIGIL_QT_MAJOR_VERSION == 5:
-        QTFORPYTHON = "PyQt5"
-    elif SIGIL_QT_MAJOR_VERSION == 6:
-        QTFORPYTHON = "PySide6"
     print("  Checking GUI modules")
     if bk.launcher_version() >= 20170227:
         try:
-            print("    Python Package: {}".format(QTFORPYTHON), end=" ")
-            from plugin_utils import QtWidgets
+            print("    Python Package: PyQt5", end=" ")
+            from PyQt5 import QtWidgets
             print("(%s) Found"%QtWidgets.__file__)
         except ImportError:
             print("Missing")
             errors += 1
 
-    if bk.launcher_version() >= 20210430:
-        try:
-            print("    Python Package: {} QtWebEngineWidgets".format(QTFORPYTHON), end=" ")
-            from plugin_utils import QtWebEngineWidgets
-            print("(%s) Found"%QtWebEngineWidgets.__file__)
-        except ImportError:
-            print("Missing")
-            errors += 1
-            
     try:
         print("    Python Package: tkinter", end=" ")
         import tkinter
