@@ -1,8 +1,8 @@
-/**
-**  Copyright (C) 2015 Kevin B. Hendricks Stratford, ON, Canada 
-**  Copyright (C) 2012 John Schember <john@nachtimwald.com>
-**  Copyright (C) 2012 Dave Heiland
-**  Copyright (C) 2009, 2010, 2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
+/***************************************************************************
+**  Copyright (C) 2015-2021 Kevin B. Hendricks Stratford, ON, Canada 
+**  Copyright (C) 2012      John Schember <john@nachtimwald.com>
+**  Copyright (C) 2012      Dave Heiland
+**  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
 **
@@ -26,11 +26,9 @@
 #include <QtCore/QHash>
 #include <QtCore/QList>
 #include <QtCore/QString>
-#include <QtCore/QXmlStreamReader>
-#include <QtWebKitWidgets/QWebFrame>
-#include <QtWebKitWidgets/QWebPage>
-#include <QtXml/QXmlInputSource>
-#include <QtXml/QXmlSimpleReader>
+#include <QXmlStreamReader>
+// #include <QtWebKitWidgets/QWebFrame>
+// #include <QtWebKitWidgets/QWebPage>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 #include <QDir>
@@ -39,6 +37,7 @@
 #include "BookManipulation/CleanSource.h"
 #include "BookManipulation/XhtmlDoc.h"
 #include "Misc/Utility.h"
+#include "Parsers/TagLister.h"
 #include "sigil_constants.h"
 #include "sigil_exception.h"
 
@@ -62,7 +61,7 @@ const QStringList ID_TAGS = QStringList() << BLOCK_LEVEL_TAGS <<
                             "font" << "i" << "label" << "mark" << "pre" << "small" <<
                             "span" << "strike" << "strong" << "sub" << "sup" << "u";
 const QStringList ANCHOR_TAGS = QStringList() << "a";
-const QStringList SRC_TAGS = QStringList() << "link" << "img";
+const QStringList SRC_TAGS = QStringList() << "img";
 
 
 const int XML_DECLARATION_SEARCH_PREFIX_SIZE = 150;
@@ -72,6 +71,9 @@ static const QString URL_ATTRIBUTE_SEARCH = ":.*(url\\s*\\([^\\)]+\\))";
 
 const QString BREAK_TAG_SEARCH  = "(<div>\\s*)?<hr\\s*class\\s*=\\s*\"[^\"]*(sigil_split_marker|sigilChapterBreak)[^\"]*\"\\s*/>(\\s*</div>)?";
 
+static const QString NEXT_TAG_LOCATION      = "<[^!>]+>";
+static const QString TAG_NAME_SEARCH        = "<\\s*([^\\s>]+)";
+
 // Resolves custom ENTITY declarations
 QString XhtmlDoc::ResolveCustomEntities(const QString &source)
 {
@@ -105,6 +107,8 @@ QString XhtmlDoc::ResolveCustomEntities(const QString &source)
 
     // ...and now replace all occurrences
     foreach(QString key, entities.keys()) {
+        // this could be dangerous so remove them to be safest
+        // new_source.replace(key, "");
         new_source.replace(key, entities[ key ]);
     }
     // Clean up what's left of the custom entity declaration field
@@ -128,14 +132,13 @@ QList<XhtmlDoc::XMLElement> XhtmlDoc::GetTagsInHead(const QString &source, const
         reader.readNext();
 
         if (reader.isStartElement()) {
-            if (reader.name() == "head" || reader.name() == "HEAD") {
+            if (reader.name().compare(QLatin1String("head"),Qt::CaseInsensitive) == 0) {
                 in_head = true;
-            } else if (in_head && reader.name() == tag_name) {
+            } else if (in_head && reader.name().compare(tag_name) == 0) {
                 matching_elements.append(CreateXMLElement(reader));
             }
         } else if (reader.isEndElement() &&
-                   (reader.name() == "head" || reader.name() == "HEAD")
-                  ) {
+                   (reader.name().compare(QLatin1String("head"),Qt::CaseInsensitive) == 0)) {
             break;
         }
     }
@@ -163,7 +166,7 @@ QList<XhtmlDoc::XMLElement> XhtmlDoc::GetTagsInDocument(const QString &source, c
         reader.readNext();
 
         if (reader.isStartElement() &&
-            reader.name() == tag_name) {
+            (reader.name().compare(tag_name) == 0)) {
             matching_elements.append(CreateXMLElement(reader));
         }
     }
@@ -260,8 +263,7 @@ QList<QString> XhtmlDoc::GetAllDescendantHrefs(const QString & source)
     return hrefs;
 }
 
-
-XhtmlDoc::WellFormedError XhtmlDoc::WellFormedErrorForSource(const QString &source, QString version)
+XhtmlDoc::WellFormedError XhtmlDoc::GumboWellFormedErrorForSource(const QString &source, QString version)
 {
     GumboInterface gi = GumboInterface(source, version);
     QList<GumboWellFormedError> results = gi.error_check();
@@ -276,105 +278,141 @@ XhtmlDoc::WellFormedError XhtmlDoc::WellFormedErrorForSource(const QString &sour
 }
 
 
+XhtmlDoc::WellFormedError XhtmlDoc::WellFormedErrorForSource(const QString &source, QString version)
+{
+    QXmlStreamReader reader(source);
+    int ndoctypes = 0;
+    int nhtmltags = 0;
+    int nheadtags = 0;
+    int nbodytags = 0;
+
+    while (!reader.atEnd()) {
+        reader.readNext();
+        if (reader.isDTD()) ndoctypes++;
+        if (reader.isStartElement()) {
+            if (reader.name().compare(QLatin1String("html")) == 0) nhtmltags++;
+            if (reader.name().compare(QLatin1String("head")) == 0) nheadtags++;
+            if (reader.name().compare(QLatin1String("body")) == 0) nbodytags++;
+        }
+    }
+    if (reader.hasError()) {
+        XhtmlDoc::WellFormedError error;
+        error.line    = reader.lineNumber();
+        error.column  = reader.columnNumber();
+        error.message = QString(reader.errorString());
+        return error;
+    }
+    // make sure basic structure in place
+    if (ndoctypes != 1) {
+        XhtmlDoc::WellFormedError error;
+        error.line    = 1;
+        error.column  = 1;
+        error.message = "Missing DOCTYPE";
+        return error;
+    }
+    if (nhtmltags != 1) {
+        XhtmlDoc::WellFormedError error;
+        error.line    = 1;
+        error.column  = 1;
+        error.message = "Missing html tag";
+        return error;
+    }
+    if (nheadtags != 1) {
+        XhtmlDoc::WellFormedError error;
+        error.line    = 1;
+        error.column  = 1;
+        error.message = "Missing head tag";
+        return error;
+    }
+    if (nbodytags != 1) {
+        XhtmlDoc::WellFormedError error;
+        error.line    = 1;
+        error.column  = 1;
+        error.message = "Missing body tag";
+        return error;
+    }
+    return XhtmlDoc::WellFormedError();
+}
+
 bool XhtmlDoc::IsDataWellFormed(const QString &data, QString version)
 {
   XhtmlDoc::WellFormedError error = XhtmlDoc::WellFormedErrorForSource(data, version);
     return error.line == -1;
 }
 
-
-// Accepts a string with HTML and returns the text
-// in that HTML fragment. For instance:
-//   <h1>Hello <b>Qt</b>&nbsp;this is great</h1>
-// returns
-//   Hello Qt this is great
-QString XhtmlDoc::GetTextInHtml(const QString &source)
-{
-    QWebPage page;
-    page.mainFrame()->setHtml(source);
-    return page.mainFrame()->toPlainText();
-}
-
-
-// Resolves HTML entities in the provided string.
-// For instance:
-//    Bonnie &amp; Clyde
-// returns
-//    Bonnie & Clyde
-QString XhtmlDoc::ResolveHTMLEntities(const QString &text)
-{
-    // Faking some HTML... this is the easiest way to do it
-    QString newsource = "<div>" + text + "</div>";
-    return GetTextInHtml(newsource);
-}
-
-
-// A tree node class without a children() function...
-// appallingly stupid, isn't it?
-QList<QWebElement> XhtmlDoc::QWebElementChildren(const QWebElement &element)
-{
-    QList<QWebElement> children;
-    const QWebElement &first_child = element.firstChild();
-
-    if (!first_child.isNull()) {
-        children.append(first_child);
-    }
-
-    QWebElement next_sibling = first_child.nextSibling();
-
-    while (!next_sibling.isNull()) {
-        children.append(next_sibling);
-        next_sibling = next_sibling.nextSibling();
-    }
-
-    return children;
-}
-
-
 QStringList XhtmlDoc::GetSGFSectionSplits(const QString &source,
         const QString &custom_header)
 {
-    QRegularExpression body_start_tag(BODY_START);
-    QRegularExpressionMatch body_start_tag_match = body_start_tag.match(source);
-    QRegularExpression body_end_tag(BODY_END);
-
-    int body_end   = source.indexOf(body_end_tag, 0);
-    int main_index = body_start_tag_match.capturedEnd();
-
-    QString header = !custom_header.isEmpty() ? custom_header + "<body>\n" : source.left(main_index);
+    
     QStringList sections;
+    TagLister taglist(source);
+
+    // abort if no body tags exist
+    int bo = taglist.findBodyOpenTag();
+    int bc = taglist.findBodyCloseTag();
+    if (bo == -1 || bc == -1) {
+        sections << source;
+        return sections;
+    }
+
+    int body_tag_start = taglist.at(bo).pos;
+    int body_tag_end   = body_tag_start + taglist.at(bo).len;
+    int body_contents_end = taglist.at(bc).pos;
+
+    QString header = source.left(body_tag_end);
+    if (!custom_header.isEmpty()) {
+        header = custom_header + "<body>\n";
+    }
+
+    QList<int>section_starts;
+    QList<int>section_ends;
+
     QRegularExpression break_tag(BREAK_TAG_SEARCH);
+    QRegularExpression match;
 
-    while (main_index != body_end) {
-        QRegularExpressionMatch match = break_tag.match(source, main_index);
-        QString body;
-
-        // We search for our HR break tag
+    // create a list of section starts and ends inside the body
+    int start_pos = body_tag_end;
+    while (start_pos < body_contents_end) {
+        QRegularExpressionMatch match = break_tag.match(source, start_pos);
         if (match.hasMatch()) {
-            // We break up the remainder of the file on the HR tag index if it's found
-            int break_index = match.capturedStart();
-            body = Utility::Substring(main_index, break_index, source);
-            main_index = break_index + match.capturedLength();
+            int split_pos = match.capturedStart();
+            if (split_pos < body_contents_end) {
+                section_starts << start_pos;
+                section_ends << split_pos;
+            }
+            start_pos = split_pos + match.capturedLength();
         } else {
-            // Otherwise, we take the rest of the file
-            body = Utility::Substring(main_index, body_end, source);
-            main_index = body_end;
+            section_starts << start_pos;
+            section_ends << body_contents_end;
+            start_pos = body_contents_end;
         }
-
-        sections.append(header + body + "</body> </html>");
+    }
+    for (int i=0; i < section_starts.size(); i++) {
+        QString text = Utility::Substring(section_starts[i], section_ends[i], source);
+        QStringList open_tag_list = GetUnmatchedTagsForPosition(section_starts[i], taglist);
+        QString open_tag_source = "";
+        if (!open_tag_list.isEmpty()) open_tag_source = open_tag_list.join(" ");
+        sections.append(header + open_tag_source + text + "</body>\n</html>\n");
+        // let gumbo/mend fill in any necessary closing tags for any open tags
+        // at the end of each section
+    }
+    // if body is empty (no split marker) then sections will be empty which should not happen
+    // handle this special case
+    if (sections.isEmpty()) {
+        sections << source;
     }
 
     return sections;
 }
 
-
+// return all links in raw encoded form
 QStringList XhtmlDoc::GetLinkedStylesheets(const QString &source)
 {
     QList<XhtmlDoc::XMLElement> link_tag_nodes;
 
     try {
         link_tag_nodes = XhtmlDoc::GetTagsInHead(source, "link");
-    } catch (ErrorParsingXml) {
+    } catch (ErrorParsingXml&) {
         // Nothing really. If we can't get the CSS style tags,
         // than that's it. No CSS returned.
     }
@@ -389,13 +427,39 @@ QStringList XhtmlDoc::GetLinkedStylesheets(const QString &source)
             element.attributes.contains("rel") &&
             (element.attributes.value("rel").toLower() == "stylesheet") &&
             element.attributes.contains("href")) {
-            linked_css_paths.append(Utility::URLDecodePath(element.attributes.value("href")));
+           linked_css_paths.append(element.attributes.value("href"));
         }
     }
     return linked_css_paths;
 }
 
 
+// return all linked javascripts in raw encoded form
+QStringList XhtmlDoc::GetLinkedJavascripts(const QString &source)
+{
+    QList<XhtmlDoc::XMLElement> script_tag_nodes;
+
+    try {
+        script_tag_nodes = XhtmlDoc::GetTagsInHead(source, "script");
+    } catch (ErrorParsingXml&) {
+        // Nothing really. If we can't get the script tags,                                                      
+        // than that's it. No scripts returned.
+    }
+
+    QStringList linked_js_paths;
+    foreach(XhtmlDoc::XMLElement element, script_tag_nodes) {
+        if (element.attributes.contains("type") &&
+            ((element.attributes.value("type").toLower() == "text/javascript") ||
+             (element.attributes.value("type").toLower() == "text/ecmascript") ||
+             (element.attributes.value("type").toLower() == "application/javascript"))
+            && element.attributes.contains("src")) {
+            linked_js_paths.append(element.attributes.value("src"));
+        }
+    }
+    return linked_js_paths;
+}
+
+
 // Returns a list of all the "visible" text nodes that are descendants
 // of the specified node. "Visible" means we ignore style tags, script tags etc...
 QList<GumboNode *> XhtmlDoc::GetVisibleTextNodes(GumboInterface &gi, GumboNode *node)
@@ -525,13 +589,30 @@ GumboNode *XhtmlDoc::GetAncestorIDElement(GumboInterface &gi, GumboNode *node)
 }
 
 
+// the returned paths are the raw href attribute values url encoded
+QStringList XhtmlDoc::GetHrefSrcPaths(const QString &source)
+{
+    QStringList destination_paths;
+    GumboInterface gi = GumboInterface(source, "any_version");
+    foreach(QString apath, gi.get_all_values_for_attribute("src")) {
+        destination_paths << apath;
+    }
+    foreach(QString apath, gi.get_all_values_for_attribute("href")) {
+        destination_paths << apath;
+    }
+    destination_paths.removeDuplicates();
+    return destination_paths;
+}
+
+
+// the returned media paths are the href attribute values url decoded
 QStringList XhtmlDoc::GetPathsToMediaFiles(const QString &source)
 {
-  QList<GumboTag> tags = QList<GumboTag>() << GIMAGE_TAGS << GVIDEO_TAGS << GAUDIO_TAGS;
-  QStringList media_paths = GetAllMediaPathsFromMediaChildren(source, tags);
-  // Remove duplicate references
-  media_paths.removeDuplicates();
-  return media_paths;
+    QList<GumboTag> tags = QList<GumboTag>() << GIMAGE_TAGS << GVIDEO_TAGS << GAUDIO_TAGS;
+    QStringList media_paths = GetAllMediaPathsFromMediaChildren(source, tags);
+    // Remove duplicate references
+    media_paths.removeDuplicates();
+    return media_paths;
 }
 
 QStringList XhtmlDoc::GetPathsToStyleFiles(const QString &source)
@@ -544,10 +625,13 @@ QStringList XhtmlDoc::GetPathsToStyleFiles(const QString &source)
         GumboNode* node = nodes.at(i);
         GumboAttribute* attr = gumbo_get_attribute(&node->v.element.attributes, "href");
         if (attr) {
-            QString relative_path = Utility::URLDecodePath(QString::fromUtf8(attr->value));
-            QFileInfo file_info(relative_path);
-            if (file_info.suffix().toLower() == "css") {
-                style_paths << relative_path;
+            QString relative_path = QString::fromUtf8(attr->value);
+            if (relative_path.indexOf(":") == -1) {
+                std::pair<QString, QString> parts = Utility::parseRelativeHREF(relative_path);
+                QFileInfo file_info(parts.first);
+                if (file_info.suffix().toLower() == "css") {
+                    style_paths << parts.first;
+                }
             }
         }
   }
@@ -596,15 +680,24 @@ QStringList XhtmlDoc::GetAllMediaPathsFromMediaChildren(const QString & source,
     QList<GumboNode*> nodes = gi.get_all_nodes_with_tags(tags);
     for (int i = 0; i < nodes.count(); ++i) {
         GumboNode* node = nodes.at(i);
+        // each element node will only hold one of the following attributes
         GumboAttribute* attr = gumbo_get_attribute(&node->v.element.attributes, "src");
         if (!attr) {
             // search for xlink:href using gumbo attribute namespace
             attr = gumbo_get_attribute(&node->v.element.attributes, "href");
             if (attr && attr->attr_namespace != GUMBO_ATTR_NAMESPACE_XLINK) attr = NULL;
         }
+        if (!attr) {
+            // search for altimg attribute from math tag
+            attr = gumbo_get_attribute(&node->v.element.attributes, "altimg");
+            if (attr && node->v.element.tag != GUMBO_TAG_MATH) attr = NULL;
+        }
         if (attr) {
-            QString relative_path = Utility::URLDecodePath(QString::fromUtf8(attr->value));
-            media_paths << relative_path;
+            QString relative_path = QString::fromUtf8(attr->value);
+            if (relative_path.indexOf(":") == -1) {
+                std::pair<QString, QString> parts = Utility::parseRelativeHREF(relative_path);
+                media_paths << parts.first;
+            }
         }
     }
     return media_paths;
@@ -635,3 +728,36 @@ XhtmlDoc::XMLElement XhtmlDoc::CreateXMLElement(QXmlStreamReader &reader)
 
     return element;
 }
+
+
+// Note for this routine to work the position must be inside the body tag someplace
+QStringList XhtmlDoc::GetUnmatchedTagsForPosition(int pos, TagLister& m_TagList)
+{
+    // Given the specified position within the body of the text, keep looking backwards finding
+    // any tags until we hit all open block tags within the body. Append all the opening tags
+    // that do not have closing tags together (ignoring self-closing tags)
+    // and return the opening tags list complete with their attributes contiguously.
+    // Note: this should *never* include the html, head, or the body opening tags
+    QStringList opening_tags;
+    QList<int> paired_tags;
+    const QString& text = m_TagList.getSource();
+    int i = m_TagList.findFirstTagOnOrAfter(pos);
+    // so start looking for unmatched tags starting at i - 1
+    i--;
+    if (i < 0) return opening_tags;
+    while((i >= 0) && (m_TagList.at(i).tname != "body")) {
+        TagLister::TagInfo ti = m_TagList.at(i);
+        if (ti.ttype == "end") {
+            paired_tags << ti.open_pos;
+        } else if (ti.ttype == "begin") {
+            if (paired_tags.contains(ti.pos)) {
+                paired_tags.removeOne(ti.pos);
+            } else {
+                opening_tags.prepend(text.mid(ti.pos, ti.len));
+            }
+        }
+        // ignore single, and all special tags like doctype, cdata, pi, xmlheaders, and comments
+        i--;
+    }
+    return opening_tags;
+}
