@@ -1,5 +1,7 @@
 /************************************************************************
 **
+**  Copyright (C) 2018-2025  Kevin Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2018-2025  Doug Massay
 **  Copyright (C) 2014  John Schember <john@nachtimwald.com>
 **
 **  This file is part of Sigil.
@@ -86,7 +88,12 @@ QString PluginDB::buildBundledInterpPath()
 #elif defined(Q_OS_WIN32)
   bundled_python3_path = QCoreApplication::applicationDirPath() + "/python3.exe";
 #else
-  bundled_python3_path = QCoreApplication::applicationDirPath() + "/python3/bin/python3";
+  if (APPIMAGE_BUILD) {
+      bundled_python3_path = QCoreApplication::applicationDirPath() + "/python3";
+  } else {
+      bundled_python3_path = "";
+  }
+ 
 #endif
   
   QFileInfo checkPython3(bundled_python3_path);
@@ -288,34 +295,43 @@ Plugin *PluginDB::load_plugin(const QString &name)
     while (!reader.atEnd()) {
         reader.readNext();
         if (reader.isStartElement()) {
-            if (reader.name() == "name") {
+            if (reader.name().compare(QLatin1String("name")) == 0) {
                 plugin->set_name(reader.readElementText());
-            } else if (reader.name() == "author") {
+            } else if (reader.name().compare(QLatin1String("author")) == 0) {
                 plugin->set_author(reader.readElementText());
-            } else if (reader.name() == "description") {
+            } else if (reader.name().compare(QLatin1String("description")) == 0) {
                 plugin->set_description(reader.readElementText());
-            } else if (reader.name() == "type") {
+            } else if (reader.name().compare(QLatin1String("type")) == 0) {
                 plugin->set_type(reader.readElementText());
-            } else if (reader.name() == "engine") {
+            } else if (reader.name().compare(QLatin1String("engine")) == 0) {
                 plugin->set_engine(reader.readElementText());
-            } else if (reader.name() == "version") {
+            } else if (reader.name().compare(QLatin1String("version")) == 0) {
                 plugin->set_version(reader.readElementText());
-            } else if (reader.name() == "oslist") {
+            } else if (reader.name().compare(QLatin1String("oslist")) == 0) {
                 plugin->set_oslist(reader.readElementText());
-            } else if (reader.name() == "autostart") {
+            } else if (reader.name().compare(QLatin1String("autostart")) == 0) {
                 plugin->set_autostart(reader.readElementText());
-            } else if (reader.name() == "autoclose") {
+            } else if (reader.name().compare(QLatin1String("autoclose")) == 0) {
                 plugin->set_autoclose(reader.readElementText());
             }
         }
     }
 
-    // look for an plugin specifc icon if one exists at 
-    // the root level of the plugin folder
-    QString iconpath = pluginsPath() + "/" + name + "/plugin.png";
-    QFileInfo fileinfo(iconpath);
-    if (fileinfo.exists() && fileinfo.isFile() && fileinfo.isReadable()) {
-        plugin->set_iconpath(iconpath);
+    // First look for a persistent custom user-specified
+    // icon in the plugin's preference folder. And then
+    // next look in the plugin folder itself for a dev
+    // supplied icon. Prefer svg versions to png versions
+    QStringList iconpaths;
+    iconpaths << pluginsPath() + "/../plugins_prefs/" + name + "/plugin.svg";
+    iconpaths << pluginsPath() + "/../plugins_prefs/" + name + "/plugin.png";
+    iconpaths << pluginsPath() + "/" + name + "/plugin.svg";
+    iconpaths << pluginsPath() + "/" + name + "/plugin.png";
+    foreach(QString ipath, iconpaths) {
+        QFileInfo iconinfo(ipath);
+        if (iconinfo.exists() && iconinfo.isFile() && iconinfo.isReadable()) {
+            plugin->set_iconpath(iconinfo.absoluteFilePath());
+            break;
+        }
     }
 
     if (!plugin->isvalid()) {
